@@ -1,4 +1,4 @@
-# Copyright 2015 Google Inc.
+# Copyright 2015 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -29,15 +29,13 @@ class Connection(_http.JSONConnection):
     :param client: The client that owns the current connection.
     """
 
-    API_BASE_URL = 'https://www.googleapis.com'
+    API_BASE_URL = "https://www.googleapis.com"
     """The base of the API call URL."""
 
-    API_VERSION = 'v2'
+    API_VERSION = "v2"
     """The version of the API, used in building the API call's URL."""
 
-    API_URL_TEMPLATE = '{api_base_url}/bigquery/{api_version}{path}'
+    API_URL_TEMPLATE = "{api_base_url}/bigquery/{api_version}{path}"
     """A template for the URL of a particular API call."""
 
-    _EXTRA_HEADERS = {
-        _http.CLIENT_INFO_HEADER: _CLIENT_INFO,
-    }
+    _EXTRA_HEADERS = {_http.CLIENT_INFO_HEADER: _CLIENT_INFO}
