@@ -1,4 +1,4 @@
-# Copyright 2017 Google Inc.
+# Copyright 2017 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,10 +15,15 @@
 """Cursor for the Google BigQuery DB-API."""
 
 import collections
-import uuid
+
+try:
+    from collections import abc as collections_abc
+except ImportError:  # Python 2.7
+    import collections as collections_abc
 
 import six
 
+from google.cloud.bigquery import job
 from google.cloud.bigquery.dbapi import _helpers
 from google.cloud.bigquery.dbapi import exceptions
 import google.cloud.exceptions
@@ -28,11 +33,17 @@ import google.cloud.exceptions
 # five are optional and are set to None if no meaningful values can be
 # provided.
 Column = collections.namedtuple(
-    'Column',
+    "Column",
     [
-        'name', 'type_code', 'display_size', 'internal_size', 'precision',
-        'scale', 'null_ok',
-    ])
+        "name",
+        "type_code",
+        "display_size",
+        "internal_size",
+        "precision",
+        "scale",
+        "null_ok",
+    ],
+)
 
 
 class Cursor(object):
@@ -41,6 +52,7 @@ class Cursor(object):
     :type connection: :class:`~google.cloud.bigquery.dbapi.Connection`
     :param connection: A DB-API connection to Google BigQuery.
     """
+
     def __init__(self, connection):
         self.connection = connection
         self.description = None
@@ -52,7 +64,7 @@ class Cursor(object):
         # a single row at a time.
         self.arraysize = 1
         self._query_data = None
-        self._query_results = None
+        self._query_job = None
 
     def close(self):
         """No-op."""
@@ -67,16 +79,20 @@ class Cursor(object):
             self.description = None
             return
 
-        self.description = tuple([
-            Column(
-                name=field.name,
-                type_code=field.field_type,
-                display_size=None,
-                internal_size=None,
-                precision=None,
-                scale=None,
-                null_ok=field.is_nullable)
-            for field in schema])
+        self.description = tuple(
+            [
+                Column(
+                    name=field.name,
+                    type_code=field.field_type,
+                    display_size=None,
+                    internal_size=None,
+                    precision=None,
+                    scale=None,
+                    null_ok=field.is_nullable,
+                )
+                for field in schema
+            ]
+        )
 
     def _set_rowcount(self, query_results):
         """Set the rowcount from query results.
@@ -86,14 +102,13 @@ class Cursor(object):
         of modified rows.
 
         :type query_results:
-            :class:`~google.cloud.bigquery.query.QueryResults`
+            :class:`~google.cloud.bigquery.query._QueryResults`
         :param query_results: results of a query
         """
         total_rows = 0
         num_dml_affected_rows = query_results.num_dml_affected_rows
 
-        if (query_results.total_rows is not None
-                and query_results.total_rows > 0):
+        if query_results.total_rows is not None and query_results.total_rows > 0:
             total_rows = query_results.total_rows
         if num_dml_affected_rows is not None and num_dml_affected_rows > 0:
             total_rows = num_dml_affected_rows
@@ -133,33 +148,30 @@ class Cursor(object):
             is generated at random.
         """
         self._query_data = None
-        self._query_results = None
+        self._query_job = None
         client = self.connection._client
-        if job_id is None:
-            job_id = str(uuid.uuid4())
 
         # The DB-API uses the pyformat formatting, since the way BigQuery does
         # query parameters was not one of the standard options. Convert both
         # the query and the parameters to the format expected by the client
         # libraries.
-        formatted_operation = _format_operation(
-            operation, parameters=parameters)
+        formatted_operation = _format_operation(operation, parameters=parameters)
         query_parameters = _helpers.to_query_parameters(parameters)
 
-        query_job = client.run_async_query(
-            job_id,
-            formatted_operation,
-            query_parameters=query_parameters)
-        query_job.use_legacy_sql = False
+        config = job.QueryJobConfig()
+        config.query_parameters = query_parameters
+        config.use_legacy_sql = False
+        self._query_job = client.query(
+            formatted_operation, job_config=config, job_id=job_id
+        )
 
         # Wait for the query to finish.
         try:
-            query_job = query_job.result()
-        except google.cloud.exceptions.GoogleCloudError:
-            raise exceptions.DatabaseError(query_job.errors)
+            self._query_job.result()
+        except google.cloud.exceptions.GoogleCloudError as exc:
+            raise exceptions.DatabaseError(exc)
 
-        query_results = query_job.query_results()
-        self._query_results = query_results
+        query_results = self._query_job._query_results
         self._set_rowcount(query_results)
         self._set_description(query_results.schema)
 
@@ -180,16 +192,27 @@ class Cursor(object):
 
         Mutates self to indicate that iteration has started.
         """
-        if self._query_results is None:
+        if self._query_job is None:
             raise exceptions.InterfaceError(
-                'No query results: execute() must be called before fetch.')
+                "No query results: execute() must be called before fetch."
+            )
 
-        if size is None:
-            size = self.arraysize
+        is_dml = (
+            self._query_job.statement_type
+            and self._query_job.statement_type.upper() != "SELECT"
+        )
+        if is_dml:
+            self._query_data = iter([])
+            return
 
         if self._query_data is None:
-            self._query_data = iter(
-                self._query_results.fetch_data(max_results=size))
+            client = self.connection._client
+            rows_iter = client.list_rows(
+                self._query_job.destination,
+                selected_fields=self._query_job._query_results.schema,
+                page_size=self.arraysize,
+            )
+            self._query_data = iter(rows_iter)
 
     def fetchone(self):
         """Fetch a single row from the results of the last ``execute*()`` call.
@@ -247,7 +270,7 @@ class Cursor(object):
             if called before ``execute()``.
         """
         self._try_fetch()
-        return [row for row in self._query_data]
+        return list(self._query_data)
 
     def setinputsizes(self, sizes):
         """No-op."""
@@ -274,7 +297,7 @@ def _format_operation_list(operation, parameters):
         if a parameter used in the operation is not found in the
         ``parameters`` argument.
     """
-    formatted_params = ['?' for _ in parameters]
+    formatted_params = ["?" for _ in parameters]
 
     try:
         return operation % tuple(formatted_params)
@@ -302,8 +325,8 @@ def _format_operation_dict(operation, parameters):
     """
     formatted_params = {}
     for name in parameters:
-        escaped_name = name.replace('`', r'\`')
-        formatted_params[name] = '@`{}`'.format(escaped_name)
+        escaped_name = name.replace("`", r"\`")
+        formatted_params[name] = "@`{}`".format(escaped_name)
 
     try:
         return operation % formatted_params
@@ -329,7 +352,7 @@ def _format_operation(operation, parameters=None):
     if parameters is None:
         return operation
 
-    if isinstance(parameters, collections.Mapping):
+    if isinstance(parameters, collections_abc.Mapping):
         return _format_operation_dict(operation, parameters)
 
     return _format_operation_list(operation, parameters)
